/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpemenubar.h>
#include <qpe/config.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qtimer.h>
#include "Fish.h"
#include <stdlib.h>
#include <time.h>

Fish::Fish(QWidget *parent, const char *name, WFlags f) :
    QMainWindow(parent, name, f)
{
    /* initialize random number generator */
    srand(::time(0));

    /* set windows style and caption */
    setCaption("Fish");

    /* create the central widget */
    m_fishWidget = new FishWidget(this);
    setCentralWidget(m_fishWidget);
    m_fishWidget->show();

    /* create the game/bubbles submenu */
    m_bubblesMenu = new QPopupMenu(0, "bubbles_menu");
    m_bubblesMenu->setCheckable(true);
    connect(m_bubblesMenu, SIGNAL(activated(int)), this, SLOT(updateBubbles(int)));
    connect(m_bubblesMenu, SIGNAL(activated(int)), m_fishWidget, SLOT(updateBubbles(int)));
    m_bubblesMenu->insertItem(tr("0"), 0);
    m_bubblesMenu->insertItem(tr("10"), 10);
    m_bubblesMenu->insertItem(tr("20"), 20);
    m_bubblesMenu->insertItem(tr("30"), 30);
    m_bubblesMenu->insertItem(tr("40"), 40);
    m_bubblesMenu->insertItem(tr("50"), 50);

    /* create the game/fish submenu */
    m_fishMenu = new QPopupMenu(0, "fish_menu");
    m_fishMenu->setCheckable(true);
    connect(m_fishMenu, SIGNAL(activated(int)), this, SLOT(updateFish(int)));
    connect(m_fishMenu, SIGNAL(activated(int)), m_fishWidget, SLOT(updateFish(int)));
    m_fishMenu->insertItem(tr("0"), 0);
    m_fishMenu->insertItem(tr("1"), 1);
    m_fishMenu->insertItem(tr("2"), 2);
    m_fishMenu->insertItem(tr("3"), 3);
    m_fishMenu->insertItem(tr("4"), 4);
    m_fishMenu->insertItem(tr("5"), 5);
    m_fishMenu->insertItem(tr("6"), 6);
    m_fishMenu->insertItem(tr("7"), 7);
    m_fishMenu->insertItem(tr("8"), 8);
    m_fishMenu->insertItem(tr("9"), 9);
    m_fishMenu->insertItem(tr("10"), 10);

    /* create the game/timer submenu */
    m_timerMenu = new QPopupMenu(0, "timer_menu");
    m_timerMenu->setCheckable(true);
    connect(m_timerMenu, SIGNAL(activated(int)), this, SLOT(updateTimer(int)));
    connect(m_timerMenu, SIGNAL(activated(int)), m_fishWidget, SLOT(updateTimer(int)));
    m_timerMenu->insertItem(tr("50"), 50);
    m_timerMenu->insertItem(tr("100"), 100);
    m_timerMenu->insertItem(tr("150"), 150);
    
    /* create the game submenu */
    QPopupMenu *gameMenu = new QPopupMenu(0, "game_menu");
    gameMenu->insertItem(tr("Bubbles"), m_bubblesMenu);
    gameMenu->insertItem(tr("Fish"), m_fishMenu);
    gameMenu->insertItem(tr("Timer"), m_timerMenu);
    gameMenu->insertSeparator();
    gameMenu->insertItem(tr("Close"), this, SLOT(close()));

    /* create the menu */
    QPEMenuBar *menu = new QPEMenuBar(this);
    menu->insertItem(tr("Game"), gameMenu);

    /* read the configuration file */
    readConfig();

    /* check the menu entries of game size and game iq submenu */
    m_bubblesMenu->setItemChecked(m_bubbles, true);
    m_fishMenu->setItemChecked(m_fish, true);
    m_timerMenu->setItemChecked(m_timer, true);
}

Fish::~Fish()
{
    writeConfig();
}

void Fish::readConfig(void)
{
    Config cfg("Fish");

    cfg.setGroup("Settings");

    int bubbles = cfg.readNumEntry("Bubbles", 20);
    if (bubbles <= 0) {
	m_bubbles = 0;
    } else if (bubbles <= 10) {
	m_bubbles = 10;
    } else if (bubbles <= 20) {
	m_bubbles = 20;
    } else if (bubbles <= 30) {
	m_bubbles = 30;
    } else if (bubbles <= 40) {
	m_bubbles = 40;
    } else {
	m_bubbles = 50;
    }
    m_fishWidget->updateBubbles(m_bubbles);

    int fish = cfg.readNumEntry("Fish", 1);
    if (fish <= 0) {
	m_fish = 0;
    } else if (fish >= 10) {
	m_fish = 10;
    } else {
	m_fish = fish;
    }
    m_fishWidget->updateFish(m_fish);

    int timer = cfg.readNumEntry("Timer", 50);
    if (timer <= 50) {
	m_timer = 50;
    } else if (timer <= 100) {
	m_timer = 100;
    } else {
	m_timer = 150;
    }
    m_fishWidget->updateTimer(m_timer);
}

void Fish::writeConfig(void)
{
    Config cfg("Fish");

    cfg.setGroup("Settings");
    cfg.writeEntry("Bubbles", m_bubbles);
    cfg.writeEntry("Fish", m_fish);
    cfg.writeEntry("Timer", m_timer);
}

void Fish::updateBubbles(int bubbles)
{
    m_bubblesMenu->setItemChecked(m_bubbles, false);
    m_bubbles = bubbles;
    m_bubblesMenu->setItemChecked(m_bubbles, true);
    writeConfig();
}

void Fish::updateFish(int fish)
{
    m_fishMenu->setItemChecked(m_fish, false);
    m_fish = fish;
    m_fishMenu->setItemChecked(m_fish, true);
    writeConfig();
}

void Fish::updateTimer(int timer)
{
    m_timerMenu->setItemChecked(m_timer, false);
    m_timer = timer;
    m_timerMenu->setItemChecked(m_timer, true);
    writeConfig();
}

