/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpeapplication.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include "FishWidget.h"
#include "SpriteCollection.h"
#include <stdlib.h>

FishWidget::FishWidget(QWidget *parent, const char *name, WFlags f) :
    QWidget(parent, name, f)
{
    m_sprites = new SpriteCollection(this);
    m_timerTick = new QTimer(this);
    connect(m_timerTick, SIGNAL(timeout()), this, SLOT(timerTick()));
    m_bubbles = 0;
    m_fish = 0;
    m_timer = 30;
    m_first_show = true;
    memset(m_bubble_x, 0, sizeof (m_bubble_x));
    memset(m_bubble_y, 0, sizeof (m_bubble_y));
    memset(m_bubble_state, 0, sizeof (m_bubble_state));
    memset(m_fish_x, 0, sizeof (m_fish_x));
    memset(m_fish_y, 0, sizeof (m_fish_y));
    memset(m_fish_dir, 0, sizeof (m_fish_dir));
    memset(m_fish_speed, 0, sizeof (m_fish_speed));
    memset(m_fish_yspeed, 0, sizeof (m_fish_yspeed));
    memset(m_fish_which, 0, sizeof (m_fish_which));
    memset(m_fish_state, 0, sizeof (m_fish_state));
    m_timerTick->start(m_timer);
}

FishWidget::~FishWidget()
{
    if (m_sprites) {
	delete m_sprites;
    }
    if (m_timerTick) {
	m_timerTick->stop();
	delete m_timerTick;
    }
}

void FishWidget::paintEvent(QPaintEvent *event)
{
    QPixmap pixmap(width(), height());
    QPainter painter_widget(this);
    QPainter painter_pixmap(&pixmap);
    int i;

    m_sprites->drawWater(painter_pixmap, 0, 0);
    for (i = 0; i < m_bubbles; i++) {
	m_sprites->drawBubble(painter_pixmap, m_bubble_x[i], m_bubble_y[i], m_bubble_state[i]);
    }
    for (i = 0; i < m_fish; i++) {
	m_sprites->drawXFish(painter_pixmap, m_fish_x[i], m_fish_y[i],
	    m_fish_which[i], m_fish_state[i], m_fish_dir[i]);
    }
    painter_pixmap.flush();
    painter_widget.drawPixmap(0, 0, pixmap);
}

void FishWidget::showEvent(QShowEvent *event)
{
    if (m_first_show) {
	int w = width();
	int h = height();
	int i;
	m_sprites->scaleWater(w, h);
	for (i = 0; i < m_bubbles; i++) {
	    m_bubble_x[i] = rand() % w;
	    m_bubble_y[i] = rand() % h;
	    m_bubble_state[i] = 500 * (rand() % 5) + (rand() % 500);
	}
	for (i = 0; i < m_fish; i++) {
	    m_fish_dir[i] = rand() % 2;
	    m_fish_speed[i] = 1 + rand() % 3;
	    m_fish_yspeed[i] = ((rand() % 100) > 75) ? rand() % 3 - 1 : 0;
	    m_fish_which[i] = rand() % m_sprites->xFishCount();
	    m_fish_x[i] = rand() % (w - m_sprites->xFishWidth(m_fish_which[i]));
	    m_fish_y[i] = rand() % (h - m_sprites->xFishHeight(m_fish_which[i]));
	    m_fish_state[i] = 500 * (rand() % 2) + (rand() % 500);
	}
	m_first_show = false;
    }
}

void FishWidget::timerTick(void)
{
    if (m_first_show) {
	return;
    }

    int i;
    for (i = 0; i < m_bubbles; i++) {
	m_bubble_y[i]--;
	if (m_bubble_y[i] < -4) {
	    m_bubble_x[i] = rand() % width();
	    m_bubble_y[i] = height() + 4;
	}
	m_bubble_state[i] = m_bubble_state[i] + 25;
	if (m_bubble_state[i] >= 2500) {
	    m_bubble_x[i] = rand() % width();
	    m_bubble_state[i] = rand() % 500;
	}
    }
    for (i = 0; i < m_fish; i++) {
	if ((rand() % 100) > 95) {
	    m_fish_yspeed[i] = ((rand() % 100) > 75) ? rand() % 3 - 1 : 0;
	}
	if ((rand() % 100) > 98) {
	    m_fish_speed[i] = 1 + rand() % 3;
	}
	m_fish_x[i] = m_fish_x[i] + (m_fish_dir[i] ? 1 : -1) * m_fish_speed[i];
	m_fish_y[i] = m_fish_y[i] + m_fish_yspeed[i];
	if (m_fish_x[i] > width() || m_fish_x[i] < (-1 * m_sprites->xFishWidth(m_fish_which[i]))
	    || m_fish_y[i] > height() || m_fish_y[i] < (-1 * m_sprites->xFishHeight(m_fish_which[i]))) {
	    m_fish_dir[i] = rand() % 2;
	    m_fish_speed[i] = 1 + rand() % 3;
	    m_fish_which[i] = rand() % m_sprites->xFishCount();
	    m_fish_x[i] = m_fish_dir[i] ? (-1 * m_sprites->xFishWidth(m_fish_which[i])) : width();
	    m_fish_y[i] = rand() % (height() - m_sprites->xFishHeight(m_fish_which[i]));
	}
	m_fish_state[i] = m_fish_state[i] + 50;
    }
    repaint(false);
}

void FishWidget::updateBubbles(int bubbles)
{
    if (bubbles < 0 || bubbles > max_bubbles) {
	return;
    }
    if (m_first_show) {
	m_bubbles = bubbles;
	return;
    }
    if (bubbles < m_bubbles) {
	m_bubbles = bubbles;
	repaint(false);
    } else if (bubbles > m_bubbles) {
	int w = width();
	int h = height();
	while (bubbles > m_bubbles) {
	    m_bubble_x[m_bubbles] = rand() % w;
	    m_bubble_y[m_bubbles] = rand() % h;
	    m_bubble_state[m_bubbles] = 500 * (rand() % 5) + (rand() % 500);
	    m_bubbles++;
	}
	repaint(false);
    }
}

void FishWidget::updateFish(int fish)
{
    if (fish < 0 || fish > max_fish) {
	return;
    }
    if (m_first_show) {
	m_fish = fish;
	return;
    }
    if (fish < m_fish) {
	m_fish = fish;
	repaint(false);
    } else if (fish > m_fish) {
	int w = width();
	int h = height();
	while (fish > m_fish) {
	    m_fish_dir[m_fish] = rand() % 2;
	    m_fish_speed[m_fish] = 1 + rand() % 3;
	    m_fish_yspeed[m_fish] = ((rand() % 100) > 75) ? rand() % 3 - 1 : 0;
	    m_fish_which[m_fish] = rand() % m_sprites->xFishCount();
	    m_fish_x[m_fish] = rand() % (w - m_sprites->xFishWidth(m_fish_which[m_fish]));
	    m_fish_y[m_fish] = rand() % (h - m_sprites->xFishHeight(m_fish_which[m_fish]));
	    m_fish_state[m_fish] = 500 * (rand() % 2) + (rand() % 500);
	    m_fish++;
	}
	repaint(false);
    }
}

void FishWidget::updateTimer(int timer)
{
    m_timer = timer;
    m_timerTick->changeInterval(timer);
}

