/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#ifndef __FISHWIDGET_HEADER__
#define __FISHWIDGET_HEADER__

#include <qwidget.h>
#include <qtimer.h>
#include "SpriteCollection.h"

class FishWidget : public QWidget
{
    Q_OBJECT

public:
    FishWidget(QWidget *parent, const char *name = 0, WFlags f = 0);
    ~FishWidget();

public:
    static const int max_bubbles = 50;
    static const int max_fish = 10;

private:
    SpriteCollection *m_sprites;
    QTimer *m_timerTick;
    int m_bubbles;
    int m_bubble_x[max_bubbles];
    int m_bubble_y[max_bubbles];
    int m_bubble_state[max_bubbles];
    int m_fish;
    int m_fish_x[max_fish];
    int m_fish_y[max_fish];
    int m_fish_which[max_fish];
    int m_fish_dir[max_fish];
    int m_fish_speed[max_fish];
    int m_fish_yspeed[max_fish];
    int m_fish_state[max_fish];
    int m_timer;
    bool m_first_show;

public slots:
    void updateBubbles(int bubbles);
    void updateFish(int fish);
    void updateTimer(int timer);
    void timerTick(void);

protected:
    void paintEvent(QPaintEvent *event);
    void showEvent(QShowEvent *event);
};

#endif // __FISHWIDGET_HEADER__

