/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qwidget.h>
#include <qbitmap.h>
#include <qimage.h>
#include "SpriteCollection.h"
#include "images/data.c"

SpriteCollection::SpriteCollection(QWidget *parent)
{
    int i;
    int j;

    m_bubble[0].loadFromData(bubble0_data_, sizeof (bubble0_data_));
    m_bubble[1].loadFromData(bubble1_data_, sizeof (bubble1_data_));
    m_bubble[2].loadFromData(bubble2_data_, sizeof (bubble2_data_));
    m_bubble[3].loadFromData(bubble3_data_, sizeof (bubble3_data_));
    m_bubble[4].loadFromData(bubble4_data_, sizeof (bubble4_data_));
    for (i = 0; i < 5; i++) {
	m_bubble[i].setMask(m_bubble[i].createHeuristicMask());
    }
    m_fish[0].loadFromData(fish0_data_, sizeof (fish0_data_));
    m_fish[1].loadFromData(fish1_data_, sizeof (fish1_data_));
    m_fish[2].loadFromData(fish2_data_, sizeof (fish2_data_));
    for (i = 0; i < 3; i++) {
	m_fish[i].setMask(m_fish[i].createHeuristicMask());
    }
    m_fish_b[0].loadFromData(fish0b_data_, sizeof (fish0b_data_));
    m_fish_b[1].loadFromData(fish1b_data_, sizeof (fish1b_data_));
    m_fish_b[2].loadFromData(fish2b_data_, sizeof (fish2b_data_));
    for (i = 0; i < 3; i++) {
	m_fish_b[i].setMask(m_fish_b[i].createHeuristicMask());
    }
    m_xfish[0][0].loadFromData(QueenAngel_0_data_, sizeof (QueenAngel_0_data_));
    m_xfish[0][1].loadFromData(QueenAngel_1_data_, sizeof (QueenAngel_1_data_));
    m_xfish[0][2].loadFromData(QueenAngel_0b_data_, sizeof (QueenAngel_0b_data_));
    m_xfish[0][3].loadFromData(QueenAngel_1b_data_, sizeof (QueenAngel_1b_data_));
    m_xfish[1][0].loadFromData(blueking2_0_data_, sizeof (blueking2_0_data_));
    m_xfish[1][1].loadFromData(blueking2_1_data_, sizeof (blueking2_1_data_));
    m_xfish[1][2].loadFromData(blueking2_0b_data_, sizeof (blueking2_0b_data_));
    m_xfish[1][3].loadFromData(blueking2_1b_data_, sizeof (blueking2_1b_data_));
    m_xfish[2][0].loadFromData(blueking_0_data_, sizeof (blueking_0_data_));
    m_xfish[2][1].loadFromData(blueking_1_data_, sizeof (blueking_1_data_));
    m_xfish[2][2].loadFromData(blueking_0b_data_, sizeof (blueking_0b_data_));
    m_xfish[2][3].loadFromData(blueking_1b_data_, sizeof (blueking_1b_data_));
    m_xfish[3][0].loadFromData(butfish_0_data_, sizeof (butfish_0_data_));
    m_xfish[3][1].loadFromData(butfish_1_data_, sizeof (butfish_1_data_));
    m_xfish[3][2].loadFromData(butfish_0b_data_, sizeof (butfish_0b_data_));
    m_xfish[3][3].loadFromData(butfish_1b_data_, sizeof (butfish_1b_data_));
    m_xfish[4][0].loadFromData(cichlid1_0_data_, sizeof (cichlid1_0_data_));
    m_xfish[4][1].loadFromData(cichlid1_1_data_, sizeof (cichlid1_1_data_));
    m_xfish[4][2].loadFromData(cichlid1_0b_data_, sizeof (cichlid1_0b_data_));
    m_xfish[4][3].loadFromData(cichlid1_1b_data_, sizeof (cichlid1_1b_data_));
    m_xfish[5][0].loadFromData(cichlid4_0_data_, sizeof (cichlid4_0_data_));
    m_xfish[5][1].loadFromData(cichlid4_1_data_, sizeof (cichlid4_1_data_));
    m_xfish[5][2].loadFromData(cichlid4_0b_data_, sizeof (cichlid4_0b_data_));
    m_xfish[5][3].loadFromData(cichlid4_1b_data_, sizeof (cichlid4_1b_data_));
    m_xfish[6][0].loadFromData(collaris_0_data_, sizeof (collaris_0_data_));
    m_xfish[6][1].loadFromData(collaris_1_data_, sizeof (collaris_1_data_));
    m_xfish[6][2].loadFromData(collaris_0b_data_, sizeof (collaris_0b_data_));
    m_xfish[6][3].loadFromData(collaris_1b_data_, sizeof (collaris_1b_data_));
    m_xfish[7][0].loadFromData(discus2_0_data_, sizeof (discus2_0_data_));
    m_xfish[7][1].loadFromData(discus2_1_data_, sizeof (discus2_1_data_));
    m_xfish[7][2].loadFromData(discus2_0b_data_, sizeof (discus2_0b_data_));
    m_xfish[7][3].loadFromData(discus2_1b_data_, sizeof (discus2_1b_data_));
    m_xfish[8][0].loadFromData(discus3_0_data_, sizeof (discus3_0_data_));
    m_xfish[8][1].loadFromData(discus3_1_data_, sizeof (discus3_1_data_));
    m_xfish[8][2].loadFromData(discus3_0b_data_, sizeof (discus3_0b_data_));
    m_xfish[8][3].loadFromData(discus3_1b_data_, sizeof (discus3_1b_data_));
    m_xfish[9][0].loadFromData(f00_0_data_, sizeof (f00_0_data_));
    m_xfish[9][1].loadFromData(f00_1_data_, sizeof (f00_1_data_));
    m_xfish[9][2].loadFromData(f00_0b_data_, sizeof (f00_0b_data_));
    m_xfish[9][3].loadFromData(f00_1b_data_, sizeof (f00_1b_data_));
    m_xfish[10][0].loadFromData(f01_0_data_, sizeof (f01_0_data_));
    m_xfish[10][1].loadFromData(f01_1_data_, sizeof (f01_1_data_));
    m_xfish[10][2].loadFromData(f01_0b_data_, sizeof (f01_0b_data_));
    m_xfish[10][3].loadFromData(f01_1b_data_, sizeof (f01_1b_data_));
    m_xfish[11][0].loadFromData(f02_0_data_, sizeof (f02_0_data_));
    m_xfish[11][1].loadFromData(f02_1_data_, sizeof (f02_1_data_));
    m_xfish[11][2].loadFromData(f02_0b_data_, sizeof (f02_0b_data_));
    m_xfish[11][3].loadFromData(f02_1b_data_, sizeof (f02_1b_data_));
    m_xfish[12][0].loadFromData(f03_0_data_, sizeof (f03_0_data_));
    m_xfish[12][1].loadFromData(f03_1_data_, sizeof (f03_1_data_));
    m_xfish[12][2].loadFromData(f03_0b_data_, sizeof (f03_0b_data_));
    m_xfish[12][3].loadFromData(f03_1b_data_, sizeof (f03_1b_data_));
    m_xfish[13][0].loadFromData(f04_0_data_, sizeof (f04_0_data_));
    m_xfish[13][1].loadFromData(f04_1_data_, sizeof (f04_1_data_));
    m_xfish[13][2].loadFromData(f04_0b_data_, sizeof (f04_0b_data_));
    m_xfish[13][3].loadFromData(f04_1b_data_, sizeof (f04_1b_data_));
    m_xfish[14][0].loadFromData(f05_0_data_, sizeof (f05_0_data_));
    m_xfish[14][1].loadFromData(f05_1_data_, sizeof (f05_1_data_));
    m_xfish[14][2].loadFromData(f05_0b_data_, sizeof (f05_0b_data_));
    m_xfish[14][3].loadFromData(f05_1b_data_, sizeof (f05_1b_data_));
    m_xfish[15][0].loadFromData(f06_0_data_, sizeof (f06_0_data_));
    m_xfish[15][1].loadFromData(f06_1_data_, sizeof (f06_1_data_));
    m_xfish[15][2].loadFromData(f06_0b_data_, sizeof (f06_0b_data_));
    m_xfish[15][3].loadFromData(f06_1b_data_, sizeof (f06_1b_data_));
    m_xfish[16][0].loadFromData(f07_0_data_, sizeof (f07_0_data_));
    m_xfish[16][1].loadFromData(f07_1_data_, sizeof (f07_1_data_));
    m_xfish[16][2].loadFromData(f07_0b_data_, sizeof (f07_0b_data_));
    m_xfish[16][3].loadFromData(f07_1b_data_, sizeof (f07_1b_data_));
    m_xfish[17][0].loadFromData(f08_0_data_, sizeof (f08_0_data_));
    m_xfish[17][1].loadFromData(f08_1_data_, sizeof (f08_1_data_));
    m_xfish[17][2].loadFromData(f08_0b_data_, sizeof (f08_0b_data_));
    m_xfish[17][3].loadFromData(f08_1b_data_, sizeof (f08_1b_data_));
    m_xfish[18][0].loadFromData(f09_0_data_, sizeof (f09_0_data_));
    m_xfish[18][1].loadFromData(f09_1_data_, sizeof (f09_1_data_));
    m_xfish[18][2].loadFromData(f09_0b_data_, sizeof (f09_0b_data_));
    m_xfish[18][3].loadFromData(f09_1b_data_, sizeof (f09_1b_data_));
    m_xfish[19][0].loadFromData(f10_0_data_, sizeof (f10_0_data_));
    m_xfish[19][1].loadFromData(f10_1_data_, sizeof (f10_1_data_));
    m_xfish[19][2].loadFromData(f10_0b_data_, sizeof (f10_0b_data_));
    m_xfish[19][3].loadFromData(f10_1b_data_, sizeof (f10_1b_data_));
    m_xfish[20][0].loadFromData(f11_0_data_, sizeof (f11_0_data_));
    m_xfish[20][1].loadFromData(f11_1_data_, sizeof (f11_1_data_));
    m_xfish[20][2].loadFromData(f11_0b_data_, sizeof (f11_0b_data_));
    m_xfish[20][3].loadFromData(f11_1b_data_, sizeof (f11_1b_data_));
    m_xfish[21][0].loadFromData(f12_0_data_, sizeof (f12_0_data_));
    m_xfish[21][1].loadFromData(f12_1_data_, sizeof (f12_1_data_));
    m_xfish[21][2].loadFromData(f12_0b_data_, sizeof (f12_0b_data_));
    m_xfish[21][3].loadFromData(f12_1b_data_, sizeof (f12_1b_data_));
    m_xfish[22][0].loadFromData(f13_0_data_, sizeof (f13_0_data_));
    m_xfish[22][1].loadFromData(f13_1_data_, sizeof (f13_1_data_));
    m_xfish[22][2].loadFromData(f13_0b_data_, sizeof (f13_0b_data_));
    m_xfish[22][3].loadFromData(f13_1b_data_, sizeof (f13_1b_data_));
    m_xfish[23][0].loadFromData(manta_0_data_, sizeof (manta_0_data_));
    m_xfish[23][1].loadFromData(manta_1_data_, sizeof (manta_1_data_));
    m_xfish[23][2].loadFromData(manta_0b_data_, sizeof (manta_0b_data_));
    m_xfish[23][3].loadFromData(manta_1b_data_, sizeof (manta_1b_data_));
    m_xfish[24][0].loadFromData(newf1_0_data_, sizeof (newf1_0_data_));
    m_xfish[24][1].loadFromData(newf1_1_data_, sizeof (newf1_1_data_));
    m_xfish[24][2].loadFromData(newf1_0b_data_, sizeof (newf1_0b_data_));
    m_xfish[24][3].loadFromData(newf1_1b_data_, sizeof (newf1_1b_data_));
    m_xfish[25][0].loadFromData(shark1_0_data_, sizeof (shark1_0_data_));
    m_xfish[25][1].loadFromData(shark1_1_data_, sizeof (shark1_1_data_));
    m_xfish[25][2].loadFromData(shark1_0b_data_, sizeof (shark1_0b_data_));
    m_xfish[25][3].loadFromData(shark1_1b_data_, sizeof (shark1_1b_data_));
    for (j = 0; j < max_xfish; j++) {
	for (i = 0; i < 4; i++) {
	    m_xfish[j][i].setMask(m_xfish[j][i].createHeuristicMask());
	}
    }
    m_water.loadFromData(water_data_, sizeof (water_data_));
}

void SpriteCollection::drawBubble(QPainter &painter, int x, int y, int state)
{
    painter.drawPixmap(x, y, m_bubble[(state / 500) % 5]);
}

void SpriteCollection::drawFish(QPainter &painter, int x, int y, int state, int dir)
{
    if (dir) {
	painter.drawPixmap(x, y, m_fish_b[(state / 500) % 3]);
    } else {
	painter.drawPixmap(x, y, m_fish[(state / 500) % 3]);
    }
}

void SpriteCollection::drawXFish(QPainter &painter, int x, int y, int which, int state, int dir)
{
    if (which >= 0 && which < max_xfish) {
	state = (state / 500) % 2;
	dir = dir ? 0 : 2;
	painter.drawPixmap(x, y, m_xfish[which][dir + state]);
    }
}
    
int SpriteCollection::xFishCount(void)
{
    return max_xfish;
}

int SpriteCollection::xFishWidth(int which)
{
    if (which >= 0 && which < max_xfish) {
	return m_xfish[which][0].width();
    } else {
	return -1;
    }
}

int SpriteCollection::xFishHeight(int which)
{
    if (which >= 0 && which < max_xfish) {
	return m_xfish[which][0].height();
    } else {
	return -1;
    }
}


void SpriteCollection::drawWater(QPainter &painter, int x, int y)
{
    painter.drawPixmap(x, y, m_water);
}

void SpriteCollection::scaleWater(int w, int h)
{
    if (w != m_water.width() || h != m_water.height()) {
	QImage image;
	image = m_water;
	m_water = image.smoothScale(w, h);
    }
}

