/*
 * Copyright (C) 2002 Robert Ernst <robert.ernst@linux-solutions.at>
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation and appearing in the file LICENSE.GPL included in the
 * packaging of this file.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See COPYING for GPL licensing information.
 *
 */

#include <qpe/qpeapplication.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <stdio.h>
#include <string.h>

static void
fishtopng(const char *header)
{
    FILE *fp;
    if ((fp = fopen(header, "r")) == NULL) {
	return;
    }

    char prefix[1024];
    char *ptr;
    strcpy(prefix, header);
    if ((ptr = strchr(prefix, '.')) != NULL) {
	*ptr = '\0';
    }

    char junk[1024];
    char name[1024];
    int value;
    int width = -1;
    int height = -1;
    int colors = -1;
    int back = -1;
    int i;
    for (i = 0; i < 4; i++) {
	    fscanf(fp, "%s %s %d\n", junk, name, &value);
	if ((ptr = strchr(name, '_')) != NULL) {
	    *ptr = '\0';
	    ptr++;
	} else {
	    ptr = name;
	}
	if (!strcmp(ptr, "width")) {
	    width = value;
	} else if (!strcmp(ptr, "height")) {
	    height = value;
	} else if (!strcmp(ptr, "colors")) {
	    colors = value;
	} else if (!strcmp(ptr, "back")) {
	    back = value;
	}
    }
    if (width < 1 || height < 1 || colors < 1 || back < 0) {
	return;
    }
// DEBUG:
    printf("name=%s (%d x %d, %d colors)\n", prefix, width, height, colors);

    QColor color[colors];
    do {
	value = fgetc(fp);
    } while (value != '{');
    for (i = 0; i < colors; i++) {
	fscanf(fp, "%d", &value);
	color[i].setRgb(value >> 8, color[i].green(), color[i].blue());
	do {
	    value = fgetc(fp);
	} while (value != ' ');
    }
    do {
	value = fgetc(fp);
    } while (value != '{');
    for (i = 0; i < colors; i++) {
	fscanf(fp, "%d", &value);
	color[i].setRgb(color[i].red(), value >> 8, color[i].blue());
	do {
	    value = fgetc(fp);
	} while (value != ' ');
    }
    do {
	value = fgetc(fp);
    } while (value != '{');
    for (i = 0; i < colors; i++) {
	fscanf(fp, "%d", &value);
	color[i].setRgb(color[i].red(), color[i].green(), value >> 8);
	do {
	    value = fgetc(fp);
	} while (value != ' ');
    }

// DEBUG:
    for (i = 0; i < colors; i++) {
	printf("-> color %d = %d, %d, %d\n", i, color[i].red(), color[i].green(), color[i].blue());
    }

    for (i = 0; i < 2; i++) {
	QPixmap pixmap(width, height);
	QPainter painter(&pixmap);
	do {
	    value = fgetc(fp);
	} while (value != '{');
	int x;
	int y;
	for (y = 0; y < height; y++) {
	    for (x = 0; x < width; x++) {
		fscanf(fp, " 0x%x", &value);
		if (value < colors) {
		    painter.setPen(color[value]);
		} else {
		    painter.setPen(QColor(0, 0, 0));
		}
		painter.drawPoint(x, y);
		do {
		    value = fgetc(fp);
		} while (value != ' ' && value != ',');
	    }
	}
	char png[1024];
	snprintf(png, sizeof (png), "%s_%d.png", prefix, i);
	painter.flush();
	pixmap.save(png, "PNG");
    }

    fclose(fp);
}

int
main(int argc, char *argv[])
{
    QPEApplication app(argc, argv);
    int i;

    for (i = 0; i < app.argc(); i++) {
	fishtopng(app.argv()[i]);
    }

    return 0;
}

